-- ---------------------------------------------
-- Copyright (c) 1997 by Gang Cheng
--
-- Package name: sadmin
-- Purpose: a database security layer for user access control and
--          authentication in Web/Oracle database applications
-- Author: Gang Cheng
-- Version history:
--     Gang Cheng  5/26/97 - create the base package based on two prototypes
--
-- -----------------------------------------------

drop package sadmin;
/
create or replace package sadmin is

/* Public functions/procedures for all users */
/*                                           */
procedure login(host varchar2,address varchar2,user varchar2); -- first user login page
-- first login check and generate unique session id
function validation(uid in varchar2,passwd in varchar2,host varchar2,
	address varchar2,user varchar2) return boolean;
--function validation(uid in varchar2,passwd in varchar2) return boolean;
-- regular group priv. check and timeout check. If grp = GROUP0, timeout check only.
function validation(uid varchar2,grp integer,op varchar2) return boolean;
function validation(uid varchar2,grp integer) return boolean;
-- get encripted sessionid for carrying over
function encode(real_uid varchar2) return varchar2;
-- generate encrpted session id
function new_sessionid(uid varchar2) return varchar2;
-- get the real (stored) user id and session id
function get_userid(uid varchar2) return varchar2;
function get_sessionid(uid varchar2) return varchar2;

-- any user can change timeout threshold
procedure change_time_index(uid varchar2);
procedure change_time(uid varchar2, newtime number);

-- any user can change own password
procedure change_passwd_index(uid varchar2);
procedure change_passwd(old varchar2, p1 varchar2,p2 varchar2,uid varchar2);

-- any user can check profile
procedure profile(uid varchar2);
procedure profile_change(email1 varchar2, name1 varchar2,
	host1 varchar2,url1 varchar2,tel1 varchar2, 
	office1 varchar2,timeout1 number,uid varchar2);
procedure history(uid varchar2);
procedure logout(uid varchar2);

cgi_bin_root constant varchar2(100) := 'http://osprey8.npac.syr.edu:1963/cgi-bin/sadmin';
cgi_bin constant varchar2(100) := cgi_bin_root||'/wow/';
cgi_bin_login constant varchar2(100) := cgi_bin_root||'/login';

-- user needs to supply this url
--HOME_URL constant varchar2(200) := cgi_bin||'sadmin.home';


-- Procedures used by ROOT for administration
procedure admin_index(uid varchar2);
procedure admin_home(uid varchar2);
procedure admin_help(uid varchar2);
procedure insert_user_index(uid varchar2);
procedure insert_user(id varchar2,email varchar2, passwd varchar2,
	name varchar2,grp integer,host varchar2,url varchar2,
	tel varchar2, office varchar2,uid varchar2);
procedure list_user(uid varchar2);
procedure list_user;
procedure delete_user_index(uid varchar2);
procedure delete_user(uid varchar2);
procedure delete_user(id varchar2,uid varchar2);
procedure modify_user_index(uid varchar2);
procedure modify_user(id varchar2,uid varchar2);
procedure modify_user(uid varchar2);
procedure modify_user(id varchar2,email1 varchar2, name1 varchar2,grp integer,
	host1 varchar2,url1 varchar2,tel1 varchar2, 
	office1 varchar2,timeout1 number,passwd1 varchar2,uid varchar2);

procedure modify_user(id varchar2,email1 varchar2, name1 varchar2,grp integer,
	host1 varchar2,url1 varchar2,tel1 varchar2, 
	office1 varchar2,timeout1 number,pw integer,passwd1 varchar2,uid varchar2);

procedure success_sign;
procedure error_sign;

procedure home(uid varchar2,passwd varchar2,host varchar2,address varchar2,user varchar2);

-- user's example
--procedure home(uid varchar2,passwd varchar2);

  default_timeout number(8,5) := 5; -- five minutes, default timeout

  GROUP0 constant integer := 0;
  GROUP1 constant integer := 1;
  GROUP2 constant integer := 2;
  GROUP3 constant integer := 3;
  GROUP4 constant integer := 4;
  GROUP5 constant integer := 5;
  GROUP6 constant integer := 6;
  GROUP7 constant integer := 7;
  GROUP8 constant integer := 8;
  ROOT constant integer := 9;
  root_host varchar2(20) := NULL; -- to be gathered from httpd server
end;
/
show errors;

create or replace package body sadmin is
   
hh_time date;
h_host users.host%TYPE := NULL;
h_grp users.gid%TYPE;
real_id varchar2(50);

-- first login page
-- A user application needs to implment HOME_URL?uid=x&passwd=y
--   where x is the user-supplied userid and y is the password.
--   in HOME_URL(uid varchar2, passwd varchar2), 
--   validation(uid,passwd) must be the first statement
--   which does login check and generates unique sesssion id.
-- In most cases, encode(uid) needs to be invoked to get the
--   encrepted session id to carry over multiple Web transactions.
procedure login(host varchar2,address varchar2,user varchar2) is
begin
        htp.p('<body bgcolor="white">');
        htp.p('<center><p>');
        htp.p('<br><font size=+3><strong>Database Login</strong></font><br>');
        htp.p('<br>This page is for authorized user only.');
        htp.p('<form action="'||cgi_bin||'sadmin.home'||'" method="POST">');
        htp.p('<center><table>');
        htp.p('<tr><td align=right>User Id :</td><td><input name="uid" value="" size="20"></td></tr>');
        htp.p('<tr><td align=right>Password :</td><td>');
	htp.p('<input type="password" name="passwd" value="" size="20"></td></tr>');
        htp.p('</table>');
        htp.p('<input type="SUBMIT" value="Login">'); 
	htp.p('<input type="RESET" value="Clear">'); 
	htp.p('<input type="hidden" name="host" value="'||host||'">'); 
	htp.p('<input type="hidden" name="address" value="'||address||'">');
	htp.p('<input type="hidden" name="user" value="'||user||'">');  
        htp.p('</form><p>'); 
        htp.p('<a href="'||cgi_bin||'sadmin.list_user'||'">'||'All User Accounts</a><hr>');   
end;

procedure home(uid varchar2,passwd varchar2,host varchar2,address varchar2,user varchar2) is
sessionid varchar2(50);
begin	
	if (not validation(uid,passwd,host,address,user)) then return; end if; -- login check
	sessionid := encode(uid);   -- get sessionid to carry over
        -- user's own starting stuff
	test.test_index(sessionid);
end;


-- login check and generate session id
function validation(uid in varchar2,passwd in varchar2,host varchar2,
	address varchar2,user varchar2) return boolean is
cursor cuser(uid varchar2,pwd varchar2) is select host,gid from users 
        where userid=uid and password=pwd;
session_id users.sid%TYPE;
begin
        if (uid IS NULL) then return true; end if;
        open cuser(uid,passwd);
        fetch cuser into h_host, h_grp;
        if (cuser%NOTFOUND) then
           close cuser;
           htp.p('<body bgcolor="#ffffff">');
	   error_sign;
           htp.p('<center>You have the wrong userid/password !');
           htp.p('Try again.</center>');
           return false;
        elsif ((h_grp = ROOT) and (h_host != NULL) and (h_host != root_host)) then
	   close cuser;
           htp.p('<body bgcolor="#ffffff">');
	   error_sign;
           htp.p('<center>You <blink>do not have the privilege</blink> to perform this DB operation.</center>');
           return false;
        end if;
        close cuser;

        /* update timestamp */
        update users set time=sysdate where userid=uid;

        /* generate a new session id for the login */
	session_id := new_sessionid(uid);
	
	/* update session table */
	insert into sessions(visit_time,userid,sid,browser,remote_host,remote_addr)
		values(sysdate,uid,session_id,user,host,address);

	/* update transaction table */
	insert into transactions(visit_time,userid,sid,action)
		values(sysdate,uid,session_id,'login from '||host);

	commit;

	/* root user goes to system handling procedures */
	if (h_grp = ROOT) then
		sadmin.admin_index(encode(uid));
		return false;  -- indicates system handles root users
	end if;
        return true;
end;

-- null operation comment
function validation(uid varchar2,grp integer) return boolean is
begin
	return(validation(uid,grp,NULL));
end;

-- uid - encrpted sessionid + userid, grp - grp id to be matched (=)
function validation(uid varchar2,grp integer,op varchar2) return boolean is
cursor cuser(u varchar2) is select * from users 
	where userid=u;
one users%ROWTYPE;
--s_id users.sid%TYPE;
s_id varchar2(50);
begin

	if (uid IS NULL) then return false; end if;
	real_id := get_userid(uid);
	s_id := get_sessionid(uid);

	open cuser(real_id);
	fetch cuser into one;

	if ((cuser%NOTFOUND) or ((grp != GROUP0) and (grp != one.gid)) or 
		(s_id != one.sid) or ((grp = ROOT) and (one.host != root_host)) ) then
           close cuser;
	   --htp.p('<body bgcolor="white">');
	   --htp.p('<center>You <blink>do not have the privilege</blink> to perform this operation.<p>');
	   --htp.p('Go back.</center>');
	   return false;
	elsif ((sysdate - one.time) > (one.timeout/1440.0)) then  
	   close cuser;
	   htp.p('<body bgcolor="white">');
	   htp.p('<center><p><font size="+6" color="red">STOP !!!</font>');
	   htp.p('<p><p><blink>Your login time is out.</blink> Last login: '||
			to_char(one.time,'mm/dd/yy hh24:mi')||' EST.');
	   htp.p('<p>You may choose to increase your timeout parameter.');
	   htp.p('<p><a href="'||cgi_bin_login||'" target="_top">Re-login again</a>');
	   htp.p('</center>');
	   return false;
	end if;
	close cuser;
	update users set time=sysdate where userid=real_id;
	if (op IS NOT NULL) then
		insert into transactions(visit_time,userid,sid,action) 
		values(sysdate,real_id,s_id,op);
	end if;
	commit;
	return true;

end;


-- generate encrpted session id
function new_sessionid(uid varchar2) return varchar2 is
session_id users.sid%TYPE;
begin	
	session_id := to_char(sysdate-2.24,'dd~hh24~sssss~mi');
       update users set sid=session_id where userid=uid;
       commit;
	return session_id;
end;


-- return encrepted (combined) real session id and user id for pages to carry 
--   out over session-less http sessions. real_uid -- real uid
function encode(real_uid varchar2) return varchar2 is
s_id users.sid%TYPE;

begin
	select sid into s_id from users where userid=real_uid;
	for i in 1..length(real_uid) loop
            s_id := concat(s_id,'~');
            s_id := concat(s_id,ascii(substr(real_uid,i,1)));
        end loop;
	return s_id;
end;

-- get the real (stored) user id
function get_userid(uid varchar2) return varchar2 is
s_id users.sid%TYPE := NULL;
tid users.sid%TYPE;
ch varchar2(10);
last boolean := FALSE;
begin
	tid := substr(uid,instr(uid,'~',1,4) + 1);
	loop
	   if (instr(tid,'~',1,1) = 0 ) then
		last := TRUE;
		ch := tid;
	   else
	        ch := substr(tid,1,instr(tid,'~',1,1)-1);
	        tid := substr(tid,instr(tid,'~',1,1) + 1);
	   end if;
	   ch := chr(ch);
	   s_id := concat(s_id, ch);
	   if (last) then 
		exit;
	   end if;
         end loop;
	return s_id;	
end;

-- get real (stored) session id to match with stored session id
function get_sessionid(uid varchar2) return varchar2 is
begin
	return (substr(uid,1,instr(uid,'~',1,4) - 1));
end;


procedure change_time_index(uid varchar2) is 
old_time number(8,5);

begin
	if (not validation(uid,GROUP0)) then return; end if; -- update timestamp only
	real_id := get_userid(uid);

	select timeout into old_time from users where userid=real_id;
	htp.p('<body bgcolor="white">');
	htp.p('<center><h2>Change Your TimeOut Threshold</h2>');	
	htp.p('<p>');
	htp.p('<form action="'||cgi_bin||'sadmin.change_time" method="POST">');
	htp.p('<table border=2 width=80%>');
	htp.p('<tr><td align="center">Timeout length (in minutes):</td>');
	htp.p('<td><input name="newtime" value="'||old_time||'" size=2>');
	htp.p('</td></tr>');
	htp.p('</table>');
	htp.p('<input type="hidden" name="uid" value="'||uid||'">');
	htp.p('<br><input type="submit" value="Change">');
	htp.p('<input type="reset" value="Reset">');
	htp.p('</form></center>');
end;


procedure change_time(uid varchar2, newtime number) is

real_id varchar2(50);
last integer := 0;
old_time number(8,5);

begin
	if (not validation(uid,GROUP0,'change timeout')) then return; end if; -- update timestamp only
	real_id := get_userid(uid);

	if (newtime IS NULL or newtime <= 0) then 
		htp.p('<body bgcolor="white">');
		error_sign;
		htp.p('<center>Incorrect input. Go back and try again.</center>');
		return;
	end if;
	select timeout into old_time from users where userid=real_id;
	update users set timeout = newtime where userid=real_id;
	commit;
		
	htp.p('<body bgcolor="white">');
	success_sign;
	htp.p('<center><p><p>Your timeout threshold has been changed <p>');
	htp.p('from <b>'||old_time||'</b> minutes to <b>'||newtime||'</b> minutes!</center>');
end;
	
procedure change_passwd_index(uid varchar2) is
one users%ROWTYPE;
begin
	if (not validation(uid,GROUP0)) then return; end if; -- update timestamp only
	real_id := get_userid(uid);

	htp.p('<body bgcolor="white">');
	htp.p('<center><h2>Change Password</h2></center>');
	select * into one from users where userid = real_id;
	htp.p('<center><hr><form action="'||cgi_bin||'sadmin.change_passwd" method=POST>');
	htp.p('<table border=1>');
	
	htp.p('<tr><td align="right">User Id: </td><td><b>'||real_id||'</b></td></tr>');
	htp.p('<tr><td align="right">Name: </td><td>'||one.name||'</td></tr>');
	htp.p('<tr><td align="right">Old Password:  </td><td><input type="password" name="old" value=""></td></tr>');
	htp.p('<tr><td align="right">New Password:  </td><td><input type="password" name="p1" value=""></td></tr>');
	htp.p('<tr><td align="right">New Password Again:  </td><td><input type="password" name="p2" value=""></td></tr>');
	htp.p('</table>');
	htp.p('<input type="hidden" name="uid" value="'||uid||'">');
	htp.p('<br><input type="SUBMIT" value="Change">'); 
	htp.p('</form></center><hr>');
end;

procedure change_passwd(old varchar2, p1 varchar2,p2 varchar2,uid varchar2) is
oldp users.password%TYPE;
begin
	if (not validation(uid,GROUP0,'change password')) then return; end if; -- update timestamp only
	real_id := get_userid(uid);
	select password into oldp from users where userid=real_id;
	htp.p('<body bgcolor="white">');
		
	if ((old IS NULL) or (p1 IS NULL) or (p2 IS NULL) or (old != oldp) or ( p1 != p2 )) then	
		error_sign;
		htp.p('<center><p>Incorrect input.');
		htp.p('<p><a href="'||cgi_bin||'sadmin.change_passwd_index?uid='||uid||'">Try again</a>.</center>');
		return;
	end if;

	update users set password=p1 where userid=real_id;
	commit;
	success_sign;
	htp.p('<center><p>Your password has been changed.</center>');
end;


-- ================================ Do not change below ============================ --

procedure admin_index(uid varchar2) is
begin
	-- Only ROOT can do this
	if (not validation(uid,ROOT)) then return; end if;
	htp.p('<title>Root User Administration Home</title>');
	htp.p('<frameset cols="25%,75%">');   
	htp.p('<frame src="'||cgi_bin||'sadmin.admin_home?uid='||                                    
		uid||'" name="left">');
        htp.p('<frame src="'||cgi_bin||'sadmin.admin_help?uid='|| 
                uid||'" name="right">');
        htp.p('</frameset>');
        htp.p('</html>');
end;	

procedure admin_help(uid varchar2) is
begin
	if (not validation(uid,ROOT)) then return; end if;
	htp.p('<body bgcolor="white">');
	htp.p('<h2>Help page</h2>');
	htp.p('Welcome to Web/Oracle Database Security Administration Package');
end;

procedure admin_home(uid varchar2) is
begin
	if (not validation(uid,ROOT)) then return; end if;
	htp.p('<body bgcolor="white">');
	htp.p('<center><font color="red" size="+2">Index</font></center><ul>');
	htp.p('<p><li><a href="'||cgi_bin||'sadmin.insert_user_index?uid='||uid||'" target="right">Create User</a>');
	htp.p('<p><li><a href="'||cgi_bin||'sadmin.list_user?uid='||uid||'" target="right">List Users</a>');
	htp.p('<p><li><a href="'||cgi_bin||'sadmin.modify_user_index?uid='||uid||'" target="right">Modify User</a>');
	htp.p('<p><li><a href="'||cgi_bin||'sadmin.delete_user_index?uid='||uid||'" target="right">Delete User</a>');
	htp.p('<p><li><a href="'||cgi_bin||'sadmin.change_passwd_index?uid='||uid||'" target="right">Change Password</a>');
	htp.p('<p><li><a href="'||cgi_bin||'sadmin.change_time_index?uid='||uid||'" target="right">Change Timeout</a>');
	htp.p('<p><li><a href="'||cgi_bin||'sadmin.profile?uid='||uid||'" target="right">User Profile</a>');
	htp.p('<p><li><a href="'||cgi_bin||'sadmin.history?uid='||uid||'" target="right">Login History</a>');
	htp.p('<p><li><a href="'||cgi_bin||'stats.summary?uid='||uid||'" target="right">Access Stats</a>');
	htp.p('<p><li><a href="'||cgi_bin||'sadmin.logout?uid='||uid||'" target="_top">Logout</a>');
	htp.p('<p><li><a href="'||cgi_bin||'sadmin.admin_help?uid='||uid||'" target="right">Help</a>');
	htp.p('</ul>');
end;

procedure insert_user_index(uid varchar2) is

begin
	-- only ROOT user can create a new user
        if (not validation(uid,ROOT)) then return; end if;

	htp.p('<body bgcolor="#ffffff">');
	htp.p('<center><h2>Insert a new DB user</h2></center>');	
	htp.p('<form action="'||cgi_bin||'sadmin.insert_user" method="POST">');
	htp.p('<center><table>');
	htp.p('<tr><td align=right>User Id :</td><td><input name="id" value="" size="20"></td></tr>');
	htp.p('<tr><td align=right>Initial Password :</td><td><input type="password" name="passwd" value="" size="20"></td></tr>');
	htp.p('<tr><td align=right>Name :</td><td><input name="name" value="" size="20"></td></tr>');
	htp.p('<tr><td align=right>Group Privilege :</td><td><select name="grp">');
	htp.p('<option value="1">Group 1');
	htp.p('<option value="2">Group 2');
	htp.p('<option value="3">Group 3');
	htp.p('<option value="4">Group 4');
	htp.p('<option value="5">Group 5');
	htp.p('<option value="6">Group 6');
	htp.p('<option value="7">Group 7');
	htp.p('<option value="8">Group 8');
        htp.p('<option value="9">DB Admin (root user)');
	htp.p('</select></td></tr>');
	htp.p('<tr><td align=right>Email (optional) :</td><td><input name="email" value="" size="20"></td></tr>');
	htp.p('<tr><td align=right>Client Host (for root user only) :</td><td><input name="host" value="" size="20"></td></tr>');
	htp.p('<tr><td align=right>URL (optional):</td><td><input name="url" value="N/A" size="20"></td></tr>');
	htp.p('<tr><td align=right>Telephone (optional):</td><td><input name="tel" value="" size="20"></td></tr>');
	htp.p('<tr><td align=right>Office (optional):</td><td><input name="office" value="" size="20"></td></tr>');
	htp.p('<input type="hidden" name="uid" value="'||uid||'">');
	htp.p('</table>');
	htp.p('<input type="submit" value="Create">'); 
	htp.p('<input type="reset" value="Reset">');
	htp.p('</form></center><hr>');  
end;

procedure insert_user(id varchar2,email varchar2, passwd varchar2,name varchar2,grp integer,
	host varchar2,url varchar2,tel varchar2, office varchar2,uid varchar2) is
cursor cuser(u varchar2) is select * from users where userid=u;
one users%ROWTYPE;
begin
	-- only ROOT user can create a new user
        if (not validation(uid,ROOT,'insert user '||id)) then return; end if;

	if ((uid is NULL) or (passwd is NULL) or (name IS NULL)) then
		htp.p('<body bgcolor="white">');
		error_sign;
		htp.p('<center>The field <b>User Id/Name/Password</b> must not be blank.');
		htp.p('<p>Go back and correct it. Try again.</center>');
		return;
	end if;
	real_id := id;

	open cuser(real_id);
	fetch cuser into one;
	if (cuser%FOUND) then
		close cuser;
		htp.p('<body bgcolor="white">');
		error_sign;
		htp.p('<center>There is already a user with the userid = '||htf.bold(real_id)||'. Use a different userid.');
		htp.p('<p>Go back and correct it. Try again.</center>');
		return;
	else
		insert into users(userid,password,email,name,gid,host,creation_time,url,tel,office,timeout)
			 values(id,passwd,email,name,grp,host,sysdate,url,tel,office,default_timeout);
		COMMIT;
	end if;
	close cuser;
	htp.p('<body bgcolor="white">');
	success_sign;
	htp.p('<center><p>A new user <b>'||real_id||'</b> has been created in the user database.</center><hr>');
	if (email IS NOT NULL) then
		htp.p('<p>You may send an email to inform <a href="mailto:'||email||'">'||email||'</a> of this new account.');
	end if;

end;

procedure list_user(uid varchar2) is

begin
	if (not validation(uid,GROUP0,'list users')) then return; end if;
	list_user;
end;

procedure list_user is
cursor cuser is select * from users order by userid;
one users%ROWTYPE;
h_grp varchar2(20);
h_name varchar2(200);
h_time varchar2(100);
h_office users.office%TYPE;
h_email users.email%TYPE;
h_tel users.tel%TYPE;
begin
	htp.p('<body bgcolor="white">');
	htp.p('<center><h2>All Database Users</h2>');
	htp.p('<table border=1>');
	htp.p('<tr bgcolor="#A8A8A8"><th>User Id<th>Name<th>Group<th>Email<th>Phone<th>Office<th>Timeout<th>Last login</tr>');
	open cuser;
	loop
		fetch cuser into one;
		exit when cuser%NOTFOUND;
		if (one.gid = ROOT) then
			h_grp := 'DB Admin.';
		else
		h_grp := 'Group '||one.gid;			
		end if;
		if (one.url IS NULL or one.url like '%N/A%') then
			h_name := one.name;
		else
			h_name := '<a href="'||one.url||'">'||one.name||'</a>';
		end if;
		if (one.time IS NOT NULL) then
				h_time := to_char(one.time,'mm/dd/yy, hh24:mi');
		else
				h_time := '<br>';
		end if;
		h_office := one.office;
		if (h_office IS NULL) then h_office := '<br>'; end if;
		h_tel := one.tel;
		if (h_tel IS NULL) then h_tel := '<br>'; end if;
		h_email := one.email;
		if (h_email IS NULL) then h_email := '<br>'; end if;
		
		if (MOD(cuser%ROWCOUNT,2) = 0) then
			htp.p('<tr bgcolor="#C0C0C0">');
		else
			htp.p('<tr>');
		end if;
		htp.p('<td>'||one.userid||'</td>');
		htp.p('<td>'||h_name||'</td>');
		htp.p('<td>'||h_grp||'</td><td>'||h_email);
		htp.p('<td>'||h_tel||'</td><td>'||h_office||'</td>');
		htp.p('<td align="center">'||one.timeout||'</td>');
		htp.p('<td>'||h_time||'</td></tr>');
	end loop;
	htp.p('</table>	Total: <b>'||cuser%ROWCOUNT||'</b></center><hr>');	
	close cuser;
end;

procedure delete_user_index(uid varchar2) is
cursor cuser(u varchar2) is select * from users where userid!=u order by userid;
total integer;
one users%ROWTYPE;
begin
	-- Only ROOT user can delete a user
	if (not validation(uid,ROOT)) then return; end if;
	real_id := get_userid(uid);
	htp.p('<body bgcolor="white"><center><h2>Delete an Existing User</h2></center><hr>');
	select count(*) into total from users;
	htp.p('<center>Total <i>'||total||'</i> database users. Which <b>one</b> to remove ?:<br>');
	open cuser(real_id);
	htp.p('<form action="'||cgi_bin||'sadmin.delete_user" METHOD="POST">');
	if (total < 10) then
		htp.p('<select name="id" size="'||(total-1)||'">');
	else
		htp.p('<select name="id" size="10">');
	end if;
	loop
		fetch cuser into one;
		exit when cuser%NOTFOUND;
		htp.p('<option value="'||one.userid||'">'||one.userid||' ('||one.name||')');
	end loop;
	htp.p('</select>');
	close cuser;		
	htp.p('<br><input type="SUBMIT" value="Delete">'); 
	htp.p('<input type="hidden" name="uid" value="'||uid||'">');
	htp.p('</form></center>');
	htp.p('<hr>');
end;

procedure delete_user(uid varchar2) is
begin
	 modify_user(uid);
end;

procedure delete_user(id varchar2,uid varchar2) is

begin
	-- Only ROOT user can delete a user
	if (not validation(uid,ROOT,'delete user '||id)) then return; end if;
	real_id := id;
	delete from users where userid=real_id;
	commit;
	htp.p('<body bgcolor="white">');
	success_sign;
	htp.p('<center><p>User <b>'||real_id||'</b> has been removed from the database.</center><hr>');
end;

procedure modify_user_index(uid varchar2) is
cursor cuser is select * from users order by userid;
total integer;
one users%ROWTYPE;
begin
	-- Only ROOT user can modify a user
	if (not validation(uid,ROOT)) then return; end if;
	htp.p('<body bgcolor="white"><center><h2>Modify an Existing User</h2></center><hr>');
	select count(*) into total from users;
	htp.p('<center>Total <i>'||total||'</i> users. Which <b>one</b> to modify ?:<br>');
	open cuser;
	htp.p('<form action="'||cgi_bin||'sadmin.modify_user" METHOD="POST">');
	if (total < 10) then
		htp.p('<select name="id" size="'||total||'">');
	else
		htp.p('<select name="id" size="10">');
	end if;
	loop
		fetch cuser into one;
		exit when cuser%NOTFOUND;
		htp.p('<option value="'||one.userid||'">'||one.userid||' ('||one.name||')');
	end loop;
	htp.p('</select>');
	close cuser;		
	htp.p('<input type="hidden" name="uid" value="'||uid||'">');
	htp.p('<br><input type="SUBMIT" value="Choose">'); 
	htp.p('</form></form>');
	htp.p('<hr>');
end;

procedure modify_user(uid varchar2) is
begin
	-- Only ROOT user can modify/delete a user
	if (not validation(uid,ROOT)) then return; end if;
	htp.p('<body bgcolor="white">');
	error_sign;
	htp.p('<center>You must select a user.');
	htp.p('Go back and Try again !</center>');
end;

procedure modify_user(id varchar2,uid varchar2) is
one users%ROWTYPE;
begin
	-- Only ROOT user can modify a user
	if (not validation(uid,ROOT)) then return; end if;
	real_id := id;
	select * into one from users where userid=real_id;
	htp.p('<body bgcolor="#ffffff">');
	htp.p('<center><h2>Modify a DB user</h2></center>');	
	htp.p('<form action="'||cgi_bin||'sadmin.modify_user" method="POST">');
	htp.p('<center><table>');
	htp.p('<tr><td align=right>User Id :</td><td><b>'||one.userid||'</b></td></tr>');
	htp.p('<input type="hidden" name="id" value="'||one.userid||'">');
	htp.p('<tr><td align=right>When created :</td><td>'||to_char(one.creation_time,'mm/dd/yy, hh24:mi')||'</td></tr>');
	--htp.p('<tr><td align=right>Initial Password :</td><td><input type="password" name="passwd1" value="" size="20"></td></tr>');
	htp.p('<tr><td align=right>Name :</td><td><input name="name1" value="'||one.name||'" size="20"></td></tr>');
	htp.p('<tr><td align=right>Group Privilege :</td><td><select name="grp" size=1>');
	for i in GROUP1 .. GROUP8 loop
		if (one.gid = i) then
			htp.p('<option value="'||i||'" selected>Group '||i);
		else
			htp.p('<option value="'||i||'">Group '||i);
		end if;
	end loop;

	if (one.gid = ROOT) then
		htp.p('<option value="'||ROOT||'" selected>DB Admin');
	else
		htp.p('<option value="'||ROOT||'">DB Admin');
	end if;
	htp.p('</select></td></tr>');

	htp.p('<tr><td align=right>Email (optional) :</td><td><input name="email1" value="'||one.email||'" size="20"></td></tr>');
	if (one.gid=ROOT) then
		htp.p('<tr><td align=right>Client Host :</td><td><input name="host1" value="'||one.host||'" size="20"></td></tr>');
	else
		htp.p('<input name="host1" type="hidden" value=""');
	end if;

	htp.p('<tr><td align=right>URL (optional):</td><td><input name="url1" value="'||one.url||'" size="20"></td></tr>');
	htp.p('<tr><td align=right>Telephone (optional):</td><td><input name="tel1" value="'||one.tel||'" size="20"></td></tr>');
	htp.p('<tr><td align=right>Office (optional):</td><td><input name="office1" value="'||one.office||'" size="20"></td></tr>');
	htp.p('<tr><td align=right>Timeout (in minutes):</td><td><input name="timeout1" value="'||one.timeout||'" size="3"></td></tr>');
	htp.p('<tr><td align=right><input type="checkbox" name="pw" value="1"> Reset Password:</td>
		<td><input name="passwd1" type="password" value="" size="20"></td></tr>');
	htp.p('<input type="hidden" name="uid" value="'||uid||'">');
	htp.p('</table>');
	htp.p('<br><input type="submit" value="Modify">'); 
	htp.p('<input type="reset" value="Reset">');
	htp.p('</form></center><hr>');  
end;


procedure modify_user(id varchar2,email1 varchar2, name1 varchar2,grp integer,
	host1 varchar2,url1 varchar2,tel1 varchar2, office1 varchar2,
	timeout1 number,passwd1 varchar2,uid varchar2) is
t users.timeout%TYPE;
begin
	-- only ROOT user can modify a user
        if (not validation(uid,ROOT,'modify user '||id)) then return; end if;

	if (name1 IS NULL) then
		htp.p('<body bgcolor="white">');
		error_sign;
		htp.p('<center>The field <b>Name</b> must not be blank.');
		htp.p('<p>Go back and correct it. Try again.</center>');
		return;
	end if;
	t := timeout1;
	if (t <= 0) then t := 5; end if;
	real_id := id;
	update users set email=email1,name=name1,gid=grp,host=host1,url=url1,
		tel=tel1,office=office1, timeout=t where userid=real_id;
	commit;
	htp.p('<body bgcolor="white">');
	success_sign;
	htp.p('<center><p>User <b>'||real_id||'</b> has been modified.</center><hr>');
end;

procedure modify_user(id varchar2,email1 varchar2, name1 varchar2,grp integer,
	host1 varchar2,url1 varchar2,tel1 varchar2, office1 varchar2,
	timeout1 number,pw integer,passwd1 varchar2,uid varchar2) is
t users.timeout%TYPE;
begin
	-- only ROOT user can modify a user
        if (not validation(uid,ROOT,'modify user '||id)) then return; end if;

	if (name1 IS NULL or passwd1 IS NULL) then
		htp.p('<body bgcolor="white">');
		error_sign;
		htp.p('<center>The field <b>Name/Password</b> must not be blank.');
		htp.p('<p>Go back and correct it. Try again.</center>');
		return;
	end if;
	t := timeout1;
	if (t <= 0) then t := 5; end if;
	real_id := id;
	update users set email=email1,name=name1,gid=grp,host=host1,url=url1,
		tel=tel1,office=office1, timeout=t, password=passwd1 where userid=real_id;
	commit;
	htp.p('<body bgcolor="white">');
	success_sign;
	htp.p('<center><p>User <b>'||real_id||'</b> has been modified.</center><hr>');
end;

procedure profile(uid varchar2) is
one users%ROWTYPE;
begin
	-- any user can view/change profile
	if (not validation(uid,GROUP0)) then return; end if;
	real_id := get_userid(uid);
	select * into one from users where userid=real_id;
	htp.p('<body bgcolor="#ffffff">');
	htp.p('<center><h2>User Profile</h2></center>');	
	htp.p('<form action="'||cgi_bin||'sadmin.profile_change" method="POST">');
	htp.p('<center><table>');
	htp.p('<tr><td align=right>User Id :</td><td><b>'||one.userid||'</b></td></tr>');
	--htp.p('<input type="hidden" name="id" value="'||one.userid||'">');
	htp.p('<tr><td align=right>When created :</td><td>'||to_char(one.creation_time,' hh24:mi, mm/dd/yy')||'</td></tr>');
	htp.p('<tr><td align="right"><a href="'||cgi_bin||'sadmin.history?uid='||uid||
		'">Last Login</a>');
	htp.p(':</td><td><i>'||to_char(one.time,'hh24:mi, mm/dd/yy')||'</i></td></tr>');
		
	htp.p('<tr><td align=right>Name :</td><td><input name="name1" value="'||one.name||'" size="20"></td></tr>');
	htp.p('<tr><td align=right>Group Privilege :</td><td>');

	if (one.gid = ROOT) then
		htp.p('DB Admin ');
	else
		htp.p('Group '||one.gid);
	end if;
	htp.p('</select></td></tr>');

	htp.p('<tr><td align=right>Email :</td><td><input name="email1" value="'||one.email||'" size="20"></td></tr>');
	if (one.gid=ROOT) then
		htp.p('<tr><td align=right>Client Host :</td><td><input name="host1" value="'||one.host||'" size="20"></td></tr>');
	else
		htp.p('<input name="host1" type="hidden" value="">');
	end if;
	htp.p('<tr><td align=right>URL:</td><td><input name="url1" value="'||one.url||'" size="20"></td></tr>');
	htp.p('<tr><td align=right>Telephone:</td><td><input name="tel1" value="'||one.tel||'" size="20"></td></tr>');
	htp.p('<tr><td align=right>Office:</td><td><input name="office1" value="'||one.office||'" size="20"></td></tr>');
	htp.p('<tr><td align=right>Timeout (in minutes):</td><td><input name="timeout1" value="'||one.timeout||'" size="3"></td></tr>');
	htp.p('<input type="hidden" name="uid" value="'||uid||'">');
	htp.p('</table>');
	htp.p('<br><input type="submit" value="Modify">'); 
	htp.p('<input type="reset" value="Reset">');
	htp.p('</form></center><hr>');  
end;

procedure profile_change(email1 varchar2, name1 varchar2,
	host1 varchar2,url1 varchar2,tel1 varchar2, 
	office1 varchar2,timeout1 number, uid varchar2) is
begin
	-- any user can change a profile
        if (not validation(uid,GROUP0,'change profile')) then return; end if;

	if (name1 IS NULL) then
		htp.p('<body bgcolor="white">');
		error_sign;
		htp.p('<center>The field <b>Name</b> must not be blank.');
		htp.p('<p>Go back and correct it. Try again.</center>');
		return;
	end if;
	real_id := get_userid(uid);
	update users set email=email1,name=name1,host=host1,url=url1,
		tel=tel1,office=office1,timeout=timeout1 where userid=real_id;
	commit;
	htp.p('<body bgcolor="white">');
	success_sign;
	htp.p('<center><p>User <b>'||real_id||'</b> Profile has been modified.</center><hr></body>');
end;

procedure history(uid varchar2) is
begin
	stats.history(uid);
end;

procedure success_sign is
begin
	htp.p('<center><h2><blink><font color="green">Success !</font></blink></h2></center>');
end;

procedure error_sign is
begin
	htp.p('<center><h2><blink><font color="RED">Error !</font></blink></h2></center>');
end;

procedure logout(uid varchar2) is
begin

	if ( not validation(uid,GROUP0,'logout')) then
		return;
	end if;	
	htp.p('<BODY bgcolor="white"><center>');
	htp.p('User <b>'||real_id||'</b> logoff at <tt>'||to_char(sysdate,'mm/dd/yy hh24:mi')
		||'</tt>');
	htp.p('<hr><a href="'||cgi_bin_login||'" target="_top">Re-login again</a><hr></center>');
	
end;

end sadmin;
/
show errors;
/
