drop package stats;
/
create or replace package stats is
procedure summary(uid varchar2);
procedure history(uid varchar2);
procedure showvisitors;
procedure visitor_today;
procedure visitors;
procedure who(host varchar2,tot number);
procedure transaction(uid varchar2);
procedure login(uid varchar2);
procedure by_date(tot number);
procedure browser(tot number);

cgi_bin constant varchar2(100) := sadmin.cgi_bin;
  
end;

/
show errors;
/
create or replace package body stats is
real_id varchar2(50);

procedure summary(uid varchar2) is

total number(8);
min_time date;
now date;
today number(5);
begin
	if ( not sadmin.validation(uid,sadmin.ROOT)) then
		htp.p('You are not allowed to access the info.');
		return;
	end if;

	select count(*) into total from sessions;
	select min(visit_time) into min_time from sessions;
	htp.title('Summary of DB Access Information');
	htp.p('<h1>Summary of Database Access Information</h1>');
	htp.p('<ul>');
	htp.p('<li>Logging Time Period: <i>'||to_char(min_time,'dd Mon yyyy hh:miam')||
		' - now ('||to_char(sysdate,'dd Mon yyyy hh:miam')||')</i>');
	htp.p('<li>Total logins: '||htf.url(cgi_bin||'stats.showvisitors',htf.bold(total)));
	select count(*) into today from sessions where visit_time >=to_date(to_char(sysdate,'ddmmyy'),'ddmmyy');
	htp.p('<li>Total Today''s logins: '||htf.url(cgi_bin||'stats.visitor_today',htf.bold(today)));
	htp.p('<li>Visitor counts by '||
		htf.url(cgi_bin||'stats.by_date?tot='||total,'day'));
	select count(distinct remote_host) into total from sessions;
	htp.p('<li>Total Unique logins (counted by IP address): '||
		htf.url(cgi_bin||'stats.visitors',htf.bold(total)));
	select count(distinct substr(browser,1,4)) into total from sessions;
	htp.p('<li>Total Web Browser Types: '||htf.url(cgi_bin||'stats.browser?tot='||total,htf.bold(total)));
	htp.p('</ul>');

end;


procedure history(uid varchar2) is
total integer;
begin
	if ( not sadmin.validation(uid,sadmin.GROUP0)) then
		return;
	end if;
	real_id := sadmin.get_userid(uid);
	htp.p('<center><h2>Your Database Access History</h2>');
	select count(*) into total from sessions where userid=real_id;
	htp.p('<table border=1>');
	htp.p('<tr><td>Total Logins<td><a href="'||cgi_bin||'stats.login?uid='||uid||'">'||total||'</a>');
	select count(*) into total from transactions where userid=real_id;
	htp.p('<tr><td>Total Transactions<td><a href="'||cgi_bin||'stats.transaction?uid='||uid||'">'||total||'</a>');
	htp.p('</table></center>');
end;

procedure login(uid varchar2) is
cursor c_visitor(u varchar2) is select * from sessions where userid=u order by visit_time desc;

begin	
	if ( not sadmin.validation(uid,sadmin.GROUP0)) then
		return;
	end if;
	real_id := sadmin.get_userid(uid);
	htp.p('<h2>Session History of '||real_id||'</h2>');
	htp.p('<body bgcolor="white">');
	htp.p('<table border=2 width=80%>');
	htp.p('<tr><th>Remote Host Machine<th>Time<th>Browser');
	for sessions in c_visitor(real_id) loop
		htp.p('<tr><td>'||sessions.remote_host||'<tt>'||'<td>'||
			to_char(sessions.visit_time,'hh24:mi mm/dd/yy')||'<td>'||
			sessions.browser||'</tt>');
	end loop;
	htp.p('</table></center>');
end;		

procedure transaction(uid varchar2) is
cursor c_visitor(u varchar2) is select * from transactions where  userid=u order by visit_time desc;

begin	
	if ( not sadmin.validation(uid,sadmin.GROUP0)) then
		return;
	end if;
	real_id := sadmin.get_userid(uid);
	htp.p('<h2>Transaction History of '||real_id||'</h2>');
	htp.p('<body bgcolor="white">');
	htp.p('<table border=2 width=80%>');
	htp.p('<tr><th>Time<th>Operation');
	for transactions in c_visitor(real_id) loop
		htp.p('<tr>'||'<tt>'||'<td>'||
			to_char(transactions.visit_time,'hh24:mi mm/dd/yy')||'<td>'||
			transactions.action||'</tt>');
	end loop;
	htp.p('</table></center>');
end;		



procedure browser(tot number) is
	cursor cbrowser is select substr(browser,1,4),count(*) from sessions 
	   group by substr(browser,1,4) order by 2 desc, 1 asc;
	btype varchar2(10);
	browser varchar2(100);
	total number(8);
begin
	htp.htitle('Web Browser Types of the User Community');
	htp.p('<ul>');
	htp.p('<li>Total Browsers: '||htf.bold(tot));
	htp.p('<ol>');
	open cbrowser;
	loop
		fetch cbrowser into btype,total;
		exit when cbrowser%NOTFOUND;
		if (replace(btype,' ') IS NULL) then
			browser := 'Unknown';
		else
			select browser into browser from sessions where 
			browser like btype||'%' and rownum=1;
			if (instr(browser,'/',1,1) >0) then
				browser := substr(browser,1,instr(browser,'/',1,1)-1);
			end if;
			if (instr(browser,' ',1,2) >0) then
				browser := substr(browser,1,instr(browser,' ',1,2));
			end if;	
		end if;			
		htp.p('<li>'||htf.bold(browser)||': '||total);
	end loop;
	htp.p('</ol>');
	close cbrowser;
 	select count(*) into total from sessions where browser like '%X%';
	htp.p('<li>Platforms Types: UNIX ('||total||') ');
	select count(*) into total from sessions where (lower(browser) like '%win%'
		or lower(browser) like '%mac%') and browser not like '%X%';
	htp.p('PC ('||total||')');
	htp.p('<ul>');	
	select count(*) into total from sessions where lower(browser) like '%sun%';
	htp.p('<li>SUN (SunOS): '||total);
	select count(*) into total from sessions where lower(browser) like '%irix%';
	htp.p('<li>SGI (IRIX): '||total);
	select count(*) into total from sessions where lower(browser) like '%aix%';
	htp.p('<li>IBM (AIX): '||total);
	select count(*) into total from sessions where lower(browser) like '%hp%';
	htp.p('<li>HP (UX): '||total);
	select count(*) into total from sessions where lower(browser) like '%osf%';
	htp.p('<li>DEC alpha (OSF): '||total);
	select count(*) into total from sessions where lower(browser) like '%linux%';
	htp.p('<li>Linux: '||total);
	select count(*) into total from sessions where lower(browser) like '%win%' 
		and browser not like '%X%';
	htp.p('<li>IBM PC (MS Window): '||total);
	select count(*) into total from sessions where lower(browser) like '%mac%';
	htp.p('<li>Macintosh: '||total);
	select count(*) into total from sessions where not (lower(browser) like '%win%'
		or lower(browser) like '%mac%' or browser like '%X%');
	htp.p('<li>Unknown platforms: '||total);
	htp.p('</ul>');
	select count(*) into total from sessions where lower(browser) like '%proxy%';
	htp.p('<li>Total browsers/machines through a firewall: '||htf.bold(total));
	htp.p('</ul>');
end;

  procedure showvisitors is
	cursor c_visitor is select userid,visit_time,remote_host,remote_addr from sessions order by visit_time desc;
	n number(7) := 0;
	begin
		select count(*) into n from sessions;
		htp.p('<h1>User Login History</h1>');
		htp.p('Total: '||htf.bold(n));
		htp.p('<body color="white">');
	 	htp.p('<center><table border=1><tr><th>No.<th>User Id<th>Access Time<th>Remote Host Machine
			<th>Remote Machine Address');
		n := n + 1;
		for sessions in c_visitor loop
		htp.p('<tr><td><tt>'||to_char(n - c_visitor%ROWCOUNT)||'<td>'||
			sessions.userid||'<td>'||
			to_char(sessions.visit_time,'hh24:mi mm/dd/yy')||'<td>'||
			sessions.remote_host||'<td>'||sessions.remote_addr||'</tt>');
		end loop;
		htp.p('</table></center>');
	end;

 procedure visitor_today is
	cursor c_visitor is select userid,visit_time,remote_host,browser from sessions
		where visit_time >=to_date(to_char(sysdate,'ddmmyy'),'ddmmyy') order by visit_time desc;
	n number(7) := 0;
	begin
		select count(*) into n from sessions
			where visit_time >=to_date(to_char(sysdate,'ddmmyy'),'ddmmyy');
		htp.p('<h1>Today''s Logins</h1>');
		htp.p('Total: '||htf.bold(n));
		htp.p('<body color="white">');
	 	htp.p('<center><table border=1><tr><th>No.<th>User Id<th>Access Time<th>Remote Host Machine
			<th>Remote Machine Address');
		n := n + 1;
		for sessions in c_visitor loop
		htp.p('<tr><td><tt>'||to_char(n - c_visitor%ROWCOUNT)||'<td>'||
			sessions.userid||'<td>'||
			to_char(sessions.visit_time,'hh24:mi mm/dd/yy')||'<td>'||
			sessions.remote_host||'<td>'||substr(sessions.browser,1,
			instr(sessions.browser,' ',1,2)-1)||'</tt>');
		end loop;
		htp.p('</table></center>');
	end;

procedure visitors is
	cursor cvisitor is select remote_host,count(*) from sessions 
		group by remote_host order by 2 desc, 1 asc;
	total number(8);
	host sessions.remote_host%TYPE;
	begin
	select count(distinct remote_host) into total from sessions;
		htp.p('<h1>Unique Logins</h1>');
		htp.p('Total: '||htf.bold(total));
	 	htp.p('<p><b>Remote Host Machine (Login Times)</b>');
 		htp.p(htf.line);
		open cvisitor;
		loop
			fetch cvisitor into host,total; 
			exit when cvisitor%NOTFOUND;
			htp.p('<tt>'||host||' ('||htf.url(cgi_bin||'stats.who?host='||host||'&tot='||total,
				htf.bold(total))||')</tt><br>');
		end loop;
		close cvisitor;
	end;
procedure who(host varchar2,tot number) is
	cursor c_visitor(ahost varchar2) is select userid,visit_time,browser from sessions 
		where remote_host = ahost order by visit_time desc;
	n number(7) := 0;
	address sessions.remote_addr%TYPE;
	begin
		n := tot;
		htp.htitle('Login Information from Host ('||host||')');
		select remote_addr into address from sessions where remote_host = host and rownum=1;
		htp.p('Total Visits: '||htf.bold(n));
		htp.p('<br>IP Address: '||htf.bold(address));
		htp.p('<body color="white">');
	 	htp.p('<center><table border=1><tr><th>No.<th>User Id<th>Access Time<th>Remote Host Machine
			<th>Remote Machine Address');
		n := n + 1;
		for sessions in c_visitor(host) loop
			htp.p('<tr><td><tt>'||to_char(n - c_visitor%ROWCOUNT)||'<td>'||
			sessions.userid||'<td>'||
			to_char(sessions.visit_time,'hh24:mi mm/dd/yy')||'<td>'||
			sessions.browser||'</tt><br>');
		end loop;
		htp.p('</table></center>');
	end;


procedure by_date(tot number) is
	cursor cdate is select to_char(new_time(visit_time,'EST','EST'),'yy-mm-dd, Day'),
		count(*) from sessions group by to_char(
		new_time(visit_time,'EST','EST'),'yy-mm-dd, Day') 
		order by 1 desc;
	dday varchar2(30);
	total number(6);
	n integer;
	dmax number(6);
	dmin number(6);
	maxday varchar2(30);
	minday varchar2(30);
begin

	htp.htitle('Login Counts By Date');
	htp.p('<ul>');
	n := 0;
	dmax := 0;
	dmin := 100000;
	open cdate;
	loop
		fetch cdate into dday,total;
		exit when cdate%NOTFOUND;
		n := n + 1;
		htp.p('<li>'||dday||': '||htf.bold(total));
		if (total > dmax) then
			dmax := total;
			maxday := dday;
		end if;
		if (total < dmin) then
			dmin := total;
			minday := dday;
		end if;
	end loop;
	close cdate;
	htp.p('</ul>');
	htp.p('<p>Total: '||tot||'<br>Average: '||ceil(tot/n)||'/day');
	htp.p('<br>Maximum: '||dmax||' ('||maxday||')<br>Minimum: '||dmin||' ('||minday||')');
end;

end stats;
/
show errors;
/

